from collections import OrderedDict
import traceback
import inject
import re
import time

from cloudshell.networking.networking_utils import validateIP
from cloudshell.networking.cisco.firmware_data.cisco_firmware_data import CiscoFirmwareData
from cloudshell.networking.operations.interfaces.configuration_operations_interface import \
    ConfigurationOperationsInterface
from cloudshell.networking.operations.interfaces.firmware_operations_interface import FirmwareOperationsInterface
from cloudshell.shell.core.context_utils import get_resource_name
from cloudshell.networking.cisco.cisco_configuration_operations import CiscoConfigurationOperations


def _get_time_stamp():
    return time.strftime("%d%m%y-%H%M%S", time.localtime())

# def _is_valid_copy_filesystem(filesystem):
#     return not re.match('bootflash$|tftp$|ftp$|harddisk$|nvram$|pram$|flash$|localhost$', filesystem) is None


class CiscoIOSXRConfigurationOperations(CiscoConfigurationOperations):
    def __init__(self):
        CiscoConfigurationOperations.__init__(self)


    def save_configuration(self, destination_host, source_filename, vrf=None):
        """Backup 'running-config' from device to provided file_system [ftp|tftp]
    	Also possible to backup config to localhost
    	:param destination_host:  tftp/ftp server where file be saved
    	:param source_filename: what file to backup - we ignore the input form the user for
    	    this param because we support only running-config.
    	:return: status message / exception
    	"""

        #source_filename = 'running-config'

        if source_filename == '':
            source_filename = 'running-config'
        if '-config' not in source_filename:
            source_filename = source_filename.lower() + '-config'

        match_data = re.search('running-config', source_filename)
        if not match_data:
            raise Exception('Cisco IOS XR', "Configuration type is empty or wrong - should be running-config ")

        if destination_host == '':
            raise Exception('Cisco IOS XR', "Destination host is empty")

        system_name = re.sub('\s+', '_', get_resource_name())
        if len(system_name) > 23:
            system_name = system_name[:23]

        destination_filename = '{0}-{1}-{2}'.format(system_name, source_filename.replace('-config', ''),
                                                            _get_time_stamp())
        self.logger.info('destination filename is {0}'.format(destination_filename))

        if len(destination_host) <= 0:
            destination_host = self._get_resource_attribute(self.resource_name, 'Backup Location')
            if len(destination_host) <= 0:
                raise Exception('Folder path and Backup Location are empty')
        if destination_host.endswith('/'):
            destination_file = destination_host + destination_filename
        else:
            destination_file = destination_host + '/' + destination_filename

            # destination_file = destination_file.replace('127.0.0.1/', 'localhost/')
        is_uploaded = self.copy(destination_file=destination_file, source_file=source_filename, vrf=vrf)
        if is_uploaded[0] is True:
            self.logger.info('Save complete')
            return '{0},'.format(destination_filename)
        else:
            self.logger.info('Save failed with an error: {0}'.format(is_uploaded[1]))
            raise Exception(is_uploaded[1])

    def restore_configuration(self, source_file, config_type , restore_method='override', vrf=None):
        """Restore configuration on device from provided configuration file
    	Restore configuration from local file system or ftp/tftp server into 'running-config'.
    	:param source_file: relative path to the file on the remote host tftp://server/sourcefile
    	:param restore_method: override current config or not
    	:param config_type: 'running' we ignore the input form the user for
    	    this param because we support only running-config.
    	:return:
    	"""

        if not re.search('append|override', restore_method.lower()):
            raise Exception('Cisco IOS XR', "Restore method is wrong! Should be Append or Override")

        #config_type = 'running-config'
        if '-config' not in config_type:
            config_type = config_type.lower() + '-config'

        self.logger.info('Start restoring device configuration from {}'.format(source_file))

        match_data = re.search('running-config', config_type)
        if not match_data:
            raise Exception('Cisco IOS XR', "Configuration type is empty or wrong - should be running-config ")

        destination_filename = match_data.group()

        if source_file == '':
            raise Exception('Cisco IOS XR', "Path is empty")

        # source_file = source_file.replace('127.0.0.1/', 'localhost/')

        # incase of override - load the configuration file over running configuration
        if restore_method.lower() == 'override':
            self.load(source_file=source_file, vrf=vrf)
            is_uploaded = (True, '')

        # incase of append - add the configuration file to the existing one using copy command.
        elif restore_method.lower() == 'append':
            is_uploaded = self.copy(source_file=source_file, destination_file=destination_filename, vrf=vrf)

        if is_uploaded[0] is False:
            raise Exception('Cisco IOS XR', is_uploaded[1])

        is_downloaded = (True, '')
        if is_downloaded[0] is True:
            return 'Finished restore configuration!'
        else:
            raise Exception('Cisco IOS XR', is_downloaded[1])


    def load(self, source_file='', vrf=None, timeout=600, retries=5):
        """load configurationon running mode on device from provided configuration file
	    :param source_file: relative path to the file on the remote host tftp://server/sourcefile
		"""

        if not source_file:
            raise Exception('Cisco IOS XR', "No source filename provided for load method!")

        expected_map = OrderedDict()

        expected_map = {
            '[\[\(][Yy]es/[Nn]o[\)\]]|\[confirm\]': lambda session: session.send_line('yes'),
            '\(y\/n\)': lambda session: session.send_line('y'),
            '[\[\(][Yy]/[Nn][\)\]]': lambda session: session.send_line('y'),
            'overwritte': lambda session: session.send_line('yes'),
            'Do you wish to proceed':lambda session: session.send_line('yes')
        }

        load_command_str = 'load {0}'.format(source_file)
        if vrf:
            load_command_str += ' vrf {0}'.format(vrf)

        # we sand the config command and the command itself
        output=self.cli.send_config_command(command=load_command_str, expected_map=expected_map,timeout = 60)

        commit = 'commit replace'

        outcommit = self.cli.send_config_command(command=commit, expected_map=expected_map,timeout =60)
        # to exit the configuration mode
        self.cli.send_command(command='')

        match_error = re.search(r" Can't assign requested address|[Ee]rror:", output)

        error_match_commit = re.search(r'(ERROR|[Ee]rror).*', outcommit)


        if match_error is not None or error_match_commit is not None:
            error_str = output[match_error.end() + 1:]
            error_str = error_str[:error_str.find('\n')]
            raise Exception('Cisco IOS XR', 'load error: ' + error_str)

    def _check_download_from_tftp(self, output):
        """Verify if file was successfully uploaded
        :param output: output from cli
        :return True or False, and success or error message
        :rtype tuple
        """

        status_match_copy = re.search(r'\[OK\]', output)
        status_match_append = re.search('Updated Commit', output)

        is_success = (status_match_copy is not None) # results of regular copy
        if not is_success:
            is_success = (status_match_append is not None)  # result of append copy

        message = 'Copy failed. Please see logs for additional info'
        if not is_success:
            match_error = re.search('%', output, re.IGNORECASE)
            if match_error:
                message = output[match_error.end():]
                message = message.split('\n')[0]

        error_match = re.search(r'(ERROR|[Ee]rror).*', output)
        if error_match:
            self.logger.error(error_match.group())
            if is_success is True:
                message = 'Copy completed with an errors. Please see logs for additional info'

        return is_success, message




from cloudshell.shell.core.driver_bootstrap import DriverBootstrap

class CiscoIOSXRBootstrap(DriverBootstrap):

    def bindings(self, binder):
        """Binding for handler"""
        binder.bind_to_provider('handler', self._config.HANDLER_CLASS)



import inject
from cloudshell.networking.cisco.cisco_connectivity_operations import CiscoConnectivityOperations
from cloudshell.networking.cisco.cisco_send_command_operations import CiscoSendCommandOperations
from cloudshell.networking.cisco.iosxr.cisco_iosxr_autoload_operations import CiscoIOSXRSNMPAutoload
from cloudshell.networking.cisco.iosxr.cisco_iosxr_configuration_operations import CiscoIOSXRConfigurationOperations

from cloudshell.networking.networking_resource_driver_interface import NetworkingResourceDriverInterface
from cloudshell.shell.core.context_utils import context_from_args
from cloudshell.shell.core.driver_bootstrap import DriverBootstrap
from cloudshell.shell.core.resource_driver_interface import ResourceDriverInterface
from cloudshell.shell.core.driver_utils import GlobalLock

import cloudshell.networking.cisco.iosxr.cisco_iosxr_configuration as driver_config


class CiscoIOSXRResourceDriver(ResourceDriverInterface, NetworkingResourceDriverInterface, GlobalLock):
    def __init__(self):
        super(CiscoIOSXRResourceDriver, self).__init__()
        bootstrap = DriverBootstrap()
        bootstrap.add_config(driver_config)
        bootstrap.initialize()

    @context_from_args
    def initialize(self, context):
        """Initialize method
        :type context: cloudshell.shell.core.context.driver_context.InitCommandContext
        """

        return 'Finished initializing'

    def cleanup(self):
        pass

    @context_from_args
    def ApplyConnectivityChanges(self, context, request):
        connectivity_operations = CiscoConnectivityOperations()
        connectivity_operations.logger.info('Start applying connectivity changes, request is: {0}'.format(str(request)))
        response = connectivity_operations.apply_connectivity_changes(request)
        connectivity_operations.logger.info('Finished applying connectivity changes, responce is: {0}'.format(str(
            response)))
        connectivity_operations.logger.info('Apply Connectivity changes completed')
        return response

    @GlobalLock.lock
    @context_from_args
    def restore(self, context, path, config_type, restore_method, vrf=None):
        """Restore selected file to the provided destination

        :param path: source config file
        :param config_type: running config
        :param restore_method: append or override methods
        """
        configuration_operations = CiscoIOSXRConfigurationOperations()
        response = configuration_operations.restore_configuration(source_file=path, restore_method=restore_method,
                                                              config_type=config_type, vrf=vrf)
        configuration_operations.logger.info('Restore completed')
        configuration_operations.logger.info(response)

    @context_from_args
    def save(self, context, destination_host, source_filename, vrf=None):
        """Save selected file to the provided destination

        :param source_filename: source file, which will be saved
        :param destination_host: destination path where file will be saved
        """

        configuration_operations = CiscoIOSXRConfigurationOperations()
        response = configuration_operations.save_configuration(destination_host, source_filename, vrf)
        configuration_operations.logger.info('Save completed')
        return response

    @context_from_args
    def get_inventory(self, context):
        """Return device structure with all standard attributes

        :return: response
        :rtype: string
        """

        autoload_operations = CiscoIOSXRSNMPAutoload()
        response = autoload_operations.discover()
        autoload_operations.logger.info('Autoload completed')
        return response

    @GlobalLock.lock
    @context_from_args
    def update_firmware(self, context, remote_host, file_path):
        """Upload and updates firmware on the resource

        :param remote_host: path to tftp:// server where firmware file is stored
        :param file_path: firmware file name
        :return: result
        :rtype: string
        """

        firmware_operations = CiscoIOSXRConfigurationOperations()
        response = firmware_operations.update_firmware(remote_host=remote_host, file_path=file_path)
        firmware_operations.logger.info(response)

    @context_from_args
    def send_custom_command(self, context, command):
        """Send custom command

        :return: result
        :rtype: string
        """

        send_command_operations = CiscoSendCommandOperations()
        response = send_command_operations.send_command(command=command)

        return response

    @context_from_args
    def send_custom_config_command(self, context, command):
        """Send custom command in configuration mode

        :return: result
        :rtype: string
        """
        send_command_operations = CiscoSendCommandOperations()
        result_str = send_command_operations.send_config_command(command=command)
        return result_str

    @context_from_args
    def shutdown(self, context):
        pass



#!/usr/bin/python
# -*- coding: utf-8 -*-
from cloudshell.shell.core.context import ResourceCommandContext, ResourceContextDetails, ReservationContextDetails,ConnectivityContext
from cloudshell.networking.cisco.iosxr.cisco_iosxr_resource_driver import CiscoIOSXRResourceDriver
import re


def create_context():
    context = ResourceCommandContext()
    context.resource = ResourceContextDetails()
    context.resource.name = 'iosxr45'
    context.reservation = ReservationContextDetails()
    context.reservation.reservation_id = 'test_id'
    context.reservation.owner_user = 'admin'
    context.reservation.owner_email = 'fake@qualisystems.com'
    context.reservation.environment_path ='admin_Environment-6-7-2016_15-25_19-07-2016_13-46-UTC'
    context.reservation.environment_name = 'admin_Environment-6-7-2016_15-25_19-07-2016_13-46-UTC'
    context.reservation.domain = 'Global'
    context.resource.attributes = {}
    context.resource.attributes['CLI Connection Type'] = 'SSH'
    context.resource.attributes['User'] = ''
    context.resource.attributes['AdminUser'] = ''
    context.resource.attributes['Console Password'] = '3M3u7nkDzxWb0aJ/IZYeWw=='
    context.resource.attributes['Password'] = 'PgkOScppedeEbHGHdzpnrw=='
    context.resource.attributes['Enable Password'] = 'PgkOScppedeEbHGHdzpnrw=='
    context.resource.address = '192.168.73.45'
    context.resource.attributes['SNMP Version'] = '2'
    context.resource.attributes['SNMP Read Community'] = 'public'
    context.resource.attributes['Model'] = ''
    context.resource.attributes['AdminPassword'] ='DxTbqlSgAVPmrDLlHvJrsA=='
    context.resource.attributes['Vendor'] = 'Cisco'
    return context


if __name__ == '__main__':
    context = create_context()
    driver = CiscoIOSXRResourceDriver()

    #response = driver.get_inventory(context)
    #res = driver.save(context, 'tftp://82.80.35.226/test', 'startup')
    #
    #res = driver.save(context, 'flash:/config_backup/','startup')
    #C:/Users/Administrator/Desktop/test
    #tftp://12.30.245.98/test/test.txt
    #res = driver.restore(context,'flash:/config_backup/vrpcfg.zip', 'startup', 'override')

    response = driver.get_inventory(context)
    #res = driver.save(context, 'tftp://82.80.35.226/test', 'startup')

    #res = driver.ApplyConnectivityChanges(context, request)
    #res=driver.update_firmware(context,'1.1.1.1','flash:/config_backup/')
    #print driver.send_custom_command(context, "display version")
    # print response



__author__ = 'Inga'
from cloudshell.networking.cisco.cisco_handler_base import CiscoHandlerBase
from cloudshell.shell.core.context_utils import get_resource_name
from cloudshell.networking.networking_utils import *
from collections import OrderedDict


class CiscoIOSXRHandler(CiscoHandlerBase):
    def __init__(self):
        CiscoHandlerBase.__init__(self)
        self.supported_os = ['IOS-XR', 'IOS XR', 'IOSXR']







import re

from cloudshell.shell.core.context_utils import get_decrypted_password_by_attribute_name_wrapper
from cloudshell.shell.core.dependency_injection.context_based_logger import get_logger_with_thread_id

DEFAULT_PROMPT = '[>#]\s*$'
ENABLE_PROMPT = '#\s*$'
CONFIG_MODE_PROMPT = '\(config.*\)#\s*$'


def send_default_actions(session):
    """Send default commands to configure/clear session outputs
    :return:
    """
    enter_enable_mode(session=session)
    session.hardware_expect('terminal length 0', ENABLE_PROMPT)
    session.hardware_expect('terminal no exec prompt timestamp', ENABLE_PROMPT)
    session.hardware_expect(ENTER_CONFIG_MODE_PROMPT_COMMAND, CONFIG_MODE_PROMPT)
    session.hardware_expect('no logging console', CONFIG_MODE_PROMPT)
    session.hardware_expect('commit', CONFIG_MODE_PROMPT)
    session.hardware_expect('', CONFIG_MODE_PROMPT)
    session.hardware_expect('exit', DEFAULT_PROMPT + '|' + ENABLE_PROMPT)


ENTER_CONFIG_MODE_PROMPT_COMMAND = 'configure terminal'
EXIT_CONFIG_MODE_PROMPT_COMMAND = 'exit'
DEFAULT_ACTIONS = send_default_actions
SUPPORTED_OS = ['IOS-XR', 'IOS XR', 'IOSXR']


def enter_enable_mode(session):
    session.hardware_expect('enable', re_string=DEFAULT_PROMPT + '|' + ENABLE_PROMPT,
                            expect_map={'[Pp]assword': lambda session: session.send_line(
                                get_decrypted_password_by_attribute_name_wrapper('Enable Password')())})
    result = session.hardware_expect('', re_string=DEFAULT_PROMPT + '|' + ENABLE_PROMPT)
    if not re.search(ENABLE_PROMPT, result):
        raise Exception('enter_enable_mode', 'Enable password is incorrect')


GET_LOGGER_FUNCTION = get_logger_with_thread_id
POOL_TIMEOUT = 300






from cloudshell.networking.cisco.autoload.cisco_generic_snmp_autoload import CiscoGenericSNMPAutoload


class CiscoIOSXRSNMPAutoload(CiscoGenericSNMPAutoload):
    def __init__(self, snmp_handler=None, logger=None, supported_os=None):
        """Basic init with injected snmp handler and logger

        :param snmp_handler:
        :param logger:
        :return:
        """

        CiscoGenericSNMPAutoload.__init__(self, snmp_handler, logger, supported_os)

    def _get_power_supply_parent_id(self, port):
        parent_id = int(self.entity_table[port]['entPhysicalContainedIn'])
        result = ''
        if parent_id in self.entity_table.keys() and 'entPhysicalClass' in self.entity_table[parent_id]:
            if self.entity_table[parent_id]['entPhysicalClass'] == 'container':
                result = (self._get_power_supply_parent_id(parent_id) +
                          self.entity_table[parent_id]['entPhysicalParentRelPos'])
        return result
